open Types
open Printf
  
(****  PRINT GRAPHVIZ SOURCE FILE ***)

let header =
"/***  SOURCE GRAPHVIZ FILE  ***

Compile with:

    dot model.gv -Tpng -o model.png

Edit this file or change the compilation option to modify the layout

***/ " (* end header *)  

let begin_model = " 
graph ObjectDiagram {
graph [rankdir= BT,overlap = false];
node [style = filled];
fontsize=10; " (* end begin_model *)  

let end_model = " } // end model "


let world_to_string ( W k ) =
  "w(" ^  ( string_of_int  k ) ^ ")" 

  
let world_to_symbName ( W k ) =
  "w_" ^  ( string_of_int  k ) 

let  propVar_to_string (P str) = str
    
let print_succRel_gv ls =
  let print_succ_gv (w1,w2) =
    printf "%s -- %s\n"   (world_to_symbName w1) (world_to_symbName w2) in
 ignore ( List.map print_succ_gv  ls )

(* ls is a list of pairs of nodes (successor relation) *)    
let get_worlds  ls =
  let add_world w ws =
    if List.mem w ws then ws else w :: ws in
  List.fold_left  ( fun  xs (w1,w2)  ->  add_world w1 (add_world w2  xs) ) [] ls 


let print_worlds_gv  worlds forcingRel =   
  let print_world_gv w  =
    let propVars =
      if List.mem_assoc w forcingRel then List.assoc w forcingRel
	else []  in
    let propVars_string  =
      List.fold_left  ( fun str p ->  (propVar_to_string p) ^ " " ^ str ) "" propVars  in
    printf "%s  [label = \"%s: %s\"]\n" (world_to_symbName w) (world_to_string w)  propVars_string
  in
  ignore (List.map print_world_gv worlds) 
      
(* main predicate to print the model in graphvis format *)  
let print_model_gv m =
  printf "%s\n"   ( header ^ "\n\n" ^ begin_model ) ;
  printf "\n// WORLDS\n"	;
    (*  List.map ( fun w -> printf "%s " (world_to_string w) ) ( get_worlds (m.succRel) ) ; *)
   print_worlds_gv  ( get_worlds (m.succRel) ) m.forcing ; 
    printf "\n// SUCC\n"  ;
  print_succRel_gv m.succRel ;
  printf "%s\n" end_model

    

(***  READ MODEL FROM CLINGO OUTPUT  ***)    
  
let rec update_forcing (w,p) ls =
  match ls with
  | [] ->  [(w, [p] )]
  | (w1,ps) :: xs   when w1 = w ->
     if List.mem p ps then ls
       else (w1, p :: ps) :: xs
  | x :: xs    ->   x :: update_forcing (w,p) xs

    
let rec values_to_model ls =
  match ls with
  | [] ->  { count_w = 0 ; count_pw = 0 ; count_atSet = 0 ; root = W(-1) ; succRel = [] ; forcing = [] ; is_optimum = false  }
  | v :: vs ->
     (
      let m = values_to_model vs in 
      match v with
      | CountW(n) -> { m with count_w = n }
      | CountPW(n) ->  { m with count_pw = n }
      | CountAtSets(n) -> { m with count_atSet = n }
      | Root (w) ->     { m with root = w }
      | Succ(w1,w2) -> { m with succRel =  (w1,w2) :: m.succRel  }
      | Forces(w, p ) -> {  m with forcing =  update_forcing (w,p)   m.forcing }
      | IsOptimum(b) -> if b then {  m with is_optimum = true } else m 	 
     )   
     


let print_world_forcing (w,ps) =
  printf "%s:"  ( world_to_string w );
  let ps_sorted =  List.sort (fun (P s1) (P s2) -> String.compare s1 s2 ) ps  in
  ignore (List.map ( fun (P str) -> printf "%s "   str) ps_sorted);
  printf "\n"
    

    
let  print_model m =
   printf("#### MODEL ####\n") ;
   printf "Number of worlds: %d\n"  m.count_w ;
   printf "Number of pworlds: %d\n"  m.count_pw ;
   printf "Number of atSets: %d\n"  m.count_atSet ;
   printf "Root: %s\n"  (world_to_string  m.root) ;
   printf "Successor relation:\n" ;
   ignore ( List.map
	      ( fun (w1,w2) ->  printf "(%s,%s) "  (world_to_string w1) (world_to_string w2)   )    m.succRel
   ) ;
  printf "\nForcing:\n" ;
  ignore ( List.map print_world_forcing   m.forcing ) ;
   printf "#############\n"
     

let value_to_str v =
  match v with
  | Succ(w1,w2) -> "succ(" ^  ( world_to_string w1 ) ^ ","  ^ ( world_to_string w2 ) ^ ")"
  | Forces(w, P s) -> "forces(" ^ ( world_to_string w ) ^ "," ^ s ^  ")"
  | CountW(n) -> "countW(" ^ ( string_of_int n ) ^ ")"
  | CountPW(n) -> "countPW(" ^ ( string_of_int n ) ^ ")"
  | CountAtSets(n) -> "countAtSets(" ^ ( string_of_int n ) ^ ")"   
  | IsOptimum(b) ->  if b then "OPTIMUM" else ""  
  | Root(w) ->   "root(" ^  ( world_to_string w) ^ ")" 
